<?php

namespace FiloBlu\Refilo\Setup;

use Blackbird\ContentManager\Model\Content;
use Blackbird\ContentManager\Model\ResourceModel\Eav\Attribute;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\UpgradeDataInterface;
use Zend_Locale;

class UpgradeData implements UpgradeDataInterface
{
    /**
     * @var EavSetupFactory
     */
    protected $_eavSetupFactory;

    /**
     * UpgradeData constructor.
     * @param EavSetupFactory $eavSetup
     */
    public function __construct(
        EavSetupFactory $eavSetup
    ) {
        $this->_eavSetupFactory = $eavSetup;
    }

    /**
     * @param ModuleDataSetupInterface $setup
     * @param ModuleContextInterface $context
     */
    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;

        $installer->getConnection()->startSetup();
        if (version_compare($context->getVersion(), '1.0.1', '<')) {
            $this->upgradeGeoData($installer);
            $this->upgradeCountryTranslations($installer);
        }

        if (version_compare($context->getVersion(), '1.0.7', '<')) {
            $this->upgradeContentAttribute($installer);
        }

        $installer->getConnection()->endSetup();
    }

    /**
     * @param ModuleDataSetupInterface $installer
     */
    public function upgradeGeoData(ModuleDataSetupInterface $installer)
    {
        $countryContinentTableName = $installer->getTable('filoblu_refilo_country_continent');
        $continentTableName = $installer->getTable('filoblu_refilo_continent');

        $countryContinentData = json_decode(file_get_contents($this->getFilePath('country_continent.json')), true);
        $continentData = json_decode(file_get_contents($this->getFilePath('continent.json')), true);

        foreach ($countryContinentData as $item) {
            $installer->getConnection()->insert($countryContinentTableName, $item);
        }

        foreach ($continentData as $item) {
            $installer->getConnection()->insert($continentTableName, $item);
        }
    }

    /**
     * @param $filename
     * @return string
     */
    public function getFilePath($filename)
    {
        return implode(
            DIRECTORY_SEPARATOR,
            [__DIR__, '..', 'Files', $filename]
        );
    }

    /**
     * @param ModuleDataSetupInterface $installer
     */
    public function upgradeCountryTranslations(ModuleDataSetupInterface $installer)
    {
        $translationTable = $installer->getTable('translation');

        $localeList = ['en_US', 'it_IT', 'fr_FR', 'de_DE', 'es_ES', 'zh_CN','ru_RU','ja_JP'];

        foreach ($localeList as $locale) {
            $list = Zend_Locale::getTranslationList('territory', $locale, 2);
            foreach ($list as $countryCode => $translation) {
                $item = [
                    "string" => "country_" . $countryCode,
                    "store_id" => 0,
                    "translate" => $translation,
                    "locale" => $locale
                ];
                $installer->getConnection()->insert($translationTable, $item);
            }
        }
    }

    /**
     * @param ModuleDataSetupInterface $installer
     */
    private function upgradeContentAttribute(ModuleDataSetupInterface $installer)
    {
        $eavSetup = $this->_eavSetupFactory->create(['setup' => $installer]);

        try {
            $eavSetup->addAttribute(Content::ENTITY, 'content_block_identifier', [
                'type' => 'varchar',
                'label' => 'Content Block Identifier',
                'visible' => true,
                'required' => false,
                'user_defined' => false,
                'searchable' => true,
                'filterable' => true,
                'comparable' => false,
                'is_global' => true,
                'global' => Attribute::SCOPE_STORE,
            ]);
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }
}
